@@ -2,7 +2,6 @@ from . import io, log
 from .aggregate import aggregate, aggregate_async
 from .backoff import asyncbackoff, asyncretry
 from .circuit_breaker import CircuitBreaker, CircuitBroken, cutout
-from .compat import context_partial
 from .context import Context, get_context
 from .counters import Statistic, get_statistics
 from .entrypoint import entrypoint, run
@@ -11,10 +10,13 @@ from .periodic import PeriodicCallback
 from .plugins import plugins
 from .pool import PoolBase
 from .process_pool import ProcessPoolExecutor
+from .recurring import (
+    RecurringCallback, StrategyException, StrategySkip, StrategyStop,
+)
 from .service import Service
 from .signal import Signal, receiver
 from .thread_pool import (
-    IteratorWrapperSeparate, ThreadPoolExecutor, sync_await,
+    IteratorWrapperSeparate, ThreadPoolExecutor, context_partial, sync_await,
     sync_wait_coroutine, threaded, threaded_iterable,
     threaded_iterable_separate, threaded_separate, wait_coroutine,
 )
@@ -35,10 +37,14 @@ __all__ = (
     "PeriodicCallback",
     "PoolBase",
     "ProcessPoolExecutor",
+    "RecurringCallback",
     "SelectResult",
     "Service",
     "Signal",
     "Statistic",
+    "StrategyException",
+    "StrategySkip",
+    "StrategyStop",
     "ThreadPoolExecutor",
     "WorkerPool",
     "aggregate",
