@@ -94,9 +94,11 @@ class QueuedKeepaliveAioHttpTransport(
 
     async def _close(self) -> Transport:
         transport = await super()._close()
+
         if inspect.iscoroutinefunction(self.connector.close()):
             await self.connection.close()
         else:
+            # noinspection PyAsyncCall
             self.connector.close()
         return transport
 
@@ -126,18 +128,13 @@ class RavenSender(Service):
 
         log.info("Starting Raven for %r", self.sentry_dsn)
 
-        handler = SentryHandler(
-            client=self.client,
-            level=self.min_level,
-        )
+        handler = SentryHandler(client=self.client, level=self.min_level)
 
         # Add filters
         for fltr in self.filters:
             handler.addFilter(fltr)
 
-        logging.getLogger().handlers.append(
-            handler,
-        )
+        logging.getLogger().handlers.append(handler)
 
     async def stop(self, *_: Any) -> None:
         transport = self.client.remote.get_transport()
